# -*- coding: utf-8 -*-

#import the module
#we use the alias np in the following
import numpy as np

#array creation
#conversion of a list to numpy array
a = np.array([1.2,2.5,3.2,1.8])
print(type(a))

#data type
print(a.dtype)

#number of dimension
print(a.ndim)

#number of rows and columns
#shape is a tuple
print(a.shape)

#number of elements
print(a.size)

#creation and implicit type
a = np.array([1,2,4])
print(a.dtype)

#creation and explicit type
a = np.array([1,2,4],dtype=float)
print(a.dtype)

#array of Booleans
b = np.array([True,False,True,True], dtype=bool)
print(b)

#the value can be a Python object (Dict for this example)
a = np.array([{"Toto":(45,2000)},{"Tata":(34,1500)}])
print(a.dtype)

#sequence of values 0,1,2..9
a = np.arange(0,10)
print(a)

#sequence of values with step = 2
a = np.arange(0,10,2)
print(a)

#evenly spaced numbers over a specified interval
a = np.linspace(0,10,5)
print(a)

#vector with 5 times the value 1
a = np.ones(shape=5)
print(a)

#more genrally
#5 times the value 3.2
a = np.full(shape=5,fill_value=3.2)
print(a)

#load the values from a data file
a = np.loadtxt("vecteur.txt",dtype=float)
print(a)

#conversion from a list structure
lst = [1.2,3.1,4.5]
print(type(lst))

a = np.asarray(lst,dtype=float)
print(type(a))
print(a)

#a is a vector
a = np.array([1.2,2.5,3.2,1.8])
print(a)

#add an element
a = np.append(a,10)
print(a)

#delete from an index
b = np.delete(a,2)
print(b)

#resize a vector
a = np.array([1,2,3])
a.resize(5)
print(a)

#concatenate 2 vectors
x = np.array([1,2,5,6])
y = np.array([2,1,7,4])
z = np.append(x,y)
print(z)

##############
#EXTRACTIONS #
##############

v = np.array([1.2,7.4,4.2,8.5,6.3])

#print all the values
print(v)
#or
print(v[:])

#get the first element
print(v[0])

#get the last element
print(v[v.size-1])

#contiguous indices
print(v[1:3])

#extreme values, first to 3 (not included)
print(v[:3])

#extreme values, 2nd to last
print(v[2:])

#negative index
print(v[-1])

#negative indices
print(v[-3:])

#using array of indices
m = np.array([1,2,3],dtype=int)
print(v[m])

#not contiguous indices
m = np.array([0,3,2],dtype=int)
print(v[m])

#the result is a numpy array
x = v[:3]
print(type(x))

#vector of Booleans
#if b is too short, the remainder is considered to False
b = np.array([False,True,False,True,False],dtype=bool)
print(v[b])

#one can use a condition for the extraction
print(v[v < 7])

#because a condition generates a vector of Booleans
b = v < 7
print(b)
print(type(b))

#one can use also the extract() function
print(np.extract(v < 7, v))

#get the max value
print(np.max(v))

#get the index of the max value
print(np.argmax(v))

#sort the values
print(np.sort(v))

#get the indices that would sort the values
print(np.argsort(v))

#unique elements of the vector
a = np.array([1,2,2,1,1,2])
print(np.unique(a))

### statistical functions ###

#mean
print(np.mean(v))

#median
print(np.median(v))

#variance
print(np.var(v))

#percentile
print(np.percentile(v,50))

#sum
print(np.sum(v))

#cumulative sum
print(np.cumsum(v))

#calculation between vectors
x = np.array([1.2,1.3,1.0])
y = np.array([2.1,0.8,1.3])

#multiplication
print(x*y)

#addition
print(x+y)

#multiplication by a scalar
print(2*x)

#comparison of two vectors
x = np.array([1,2,5,6])
y = np.array([2,1,7,4])
b = x > y
print(b)

#logical operations
a = np.array([True,True,False,True],dtype=bool)
b = np.array([True,False,True,False],dtype=bool)

#AND
np.logical_and(a,b)

#exclusive OR
np.logical_xor(a,b)

### matrix library ###
x = np.array([1.2,1.3,1.0])
y = np.array([2.1,0.8,1.3])

#dor product
z = np.vdot(x,y)
print(z)
#or, equivalently
print(np.sum(x*y))

#vector norm
n = np.linalg.norm(x)
print(n)
#or
import math
print(math.sqrt(np.sum(x**2)))

### set routines ###
x = np.array([1,2,5,6])
y = np.array([2,1,7,4])

#intersection
print(np.intersect1d(x,y))

#union - this is not a concatenation of the vectors
print(np.union1d(x,y))

#difference - values in x but not in y
print(np.setdiff1d(x,y))